@@ -1,13 +1,17 @@
-use crate::def::NSTDURange;
-use core::{ffi::c_void, ptr};
+use crate::{
+    def::{NSTDAny, NSTDAnyConst, NSTDBool},
+    pointer::NSTDPointer,
+    range::NSTDURange,
+};
 
 /// Represents a view into a sequence of data.
 #[repr(C)]
-#[derive(Clone, Copy, Debug, Hash, PartialEq, Eq, PartialOrd, Ord)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq, PartialOrd, Ord)]
 pub struct NSTDSlice {
+    /// The number of elements the slice contains.
     pub size: usize,
-    pub element_size: usize,
-    pub data: *mut u8,
+    /// A pointer to the data.
+    pub ptr: NSTDPointer,
 }
 
 /// Access methods.
@@ -15,13 +19,13 @@ impl NSTDSlice {
     /// Gets the number of used bytes for this slice.
     #[inline]
     pub fn byte_count(&self) -> usize {
-        self.size * self.element_size
+        self.size * self.ptr.size
     }
 
     /// Returns a pointer to the end of the slice.
     #[inline]
     pub unsafe fn end_unchecked(&self) -> *mut u8 {
-        self.data.add(self.byte_count())
+        self.ptr.raw.add(self.byte_count()).cast()
     }
 }
 
@@ -30,13 +34,13 @@ impl NSTDSlice {
     /// Returns the NSTDSlice as a byte slice.
     #[inline]
     pub unsafe fn as_byte_slice(&self) -> &[u8] {
-        core::slice::from_raw_parts(self.data, self.byte_count())
+        core::slice::from_raw_parts(self.ptr.raw.cast(), self.byte_count())
     }
 
     /// Returns the NSTDSlice as a mutable byte slice.
     #[inline]
     pub unsafe fn as_byte_slice_mut(&mut self) -> &mut [u8] {
-        core::slice::from_raw_parts_mut(self.data, self.byte_count())
+        core::slice::from_raw_parts_mut(self.ptr.raw.cast(), self.byte_count())
     }
 }
 
@@ -44,19 +48,18 @@ impl NSTDSlice {
 /// Parameters:
 ///     `const NSTDUSize size` - Number of elements to slice.
 ///     `const NSTDUSize element_size` - Size of each element.
-///     `void *const data` - Pointer to the raw data.
+///     `NSTDAny data` - Pointer to the raw data.
 /// Returns: `NSTDSlice slice` - The new slice.
 #[inline]
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_core_slice_new(
     size: usize,
     element_size: usize,
-    data: *mut c_void,
+    data: NSTDAny,
 ) -> NSTDSlice {
     NSTDSlice {
         size,
-        element_size,
-        data: data as *mut u8,
+        ptr: crate::pointer::nstd_core_pointer_new(data, element_size),
     }
 }
 
@@ -67,13 +70,13 @@ pub unsafe extern "C" fn nstd_core_slice_new(
 /// Parameters:
 ///     `const NSTDSlice *const slice` - The slice.
 ///     `const NSTDUSize pos` - The position of the element to get.
-/// Returns: `void *element` - Pointer to the element.
+/// Returns: `NSTDAny element` - Pointer to the element.
 #[inline]
 #[cfg_attr(feature = "clib", no_mangle)]
-pub unsafe extern "C" fn nstd_core_slice_get(slice: &NSTDSlice, pos: usize) -> *mut c_void {
+pub unsafe extern "C" fn nstd_core_slice_get(slice: &NSTDSlice, pos: usize) -> NSTDAny {
     match slice.size > pos {
-        true => slice.data.add(pos * slice.element_size) as *mut c_void,
-        false => ptr::null_mut(),
+        true => slice.ptr.raw.add(pos * slice.ptr.size),
+        false => core::ptr::null_mut(),
     }
 }
 
@@ -81,13 +84,13 @@ pub unsafe extern "C" fn nstd_core_slice_get(slice: &NSTDSlice, pos: usize) -> *
 /// NOTE: This function follows the same behaviour rules as `nstd_core_slice_get`.
 /// Parameters:
 ///     `const NSTDSlice *const slice` - The slice.
-/// Returns: `void *element` - Pointer to the first element.
+/// Returns: `NSTDAny element` - Pointer to the first element.
 #[inline]
 #[cfg_attr(feature = "clib", no_mangle)]
-pub unsafe extern "C" fn nstd_core_slice_first(slice: &NSTDSlice) -> *mut c_void {
+pub unsafe extern "C" fn nstd_core_slice_first(slice: &NSTDSlice) -> NSTDAny {
     match slice.size > 0 {
-        true => slice.data as *mut c_void,
-        false => ptr::null_mut(),
+        true => slice.ptr.raw,
+        false => core::ptr::null_mut(),
     }
 }
 
@@ -95,54 +98,68 @@ pub unsafe extern "C" fn nstd_core_slice_first(slice: &NSTDSlice) -> *mut c_void
 /// NOTE: This function follows the same behaviour rules as `nstd_core_slice_get`.
 /// Parameters:
 ///     `const NSTDSlice *const slice` - The slice.
-/// Returns: `void *element` - Pointer to the last element.
+/// Returns: `NSTDAny element` - Pointer to the last element.
 #[inline]
 #[cfg_attr(feature = "clib", no_mangle)]
-pub unsafe extern "C" fn nstd_core_slice_last(slice: &NSTDSlice) -> *mut c_void {
+pub unsafe extern "C" fn nstd_core_slice_last(slice: &NSTDSlice) -> NSTDAny {
     match slice.size > 0 {
-        true => slice.end_unchecked().sub(slice.element_size) as *mut c_void,
-        false => ptr::null_mut(),
+        true => slice.end_unchecked().sub(slice.ptr.size).cast(),
+        false => core::ptr::null_mut(),
     }
 }
 
+/// Checks if two slices carry the same data.
+/// Parameters:
+///     `const NSTDSlice *const s1` - The first slice.
+///     `const NSTDSlice *const s2` - The second slice.
+/// Returns: `NSTDBool is_same` - True if the two slices carry the same data.
+#[inline]
+#[cfg_attr(feature = "clib", no_mangle)]
+pub unsafe extern "C" fn nstd_core_slice_compare(s1: &NSTDSlice, s2: &NSTDSlice) -> NSTDBool {
+    if s1.size == s2.size {
+        return NSTDBool::from(s1.as_byte_slice() == s2.as_byte_slice());
+    }
+    NSTDBool::NSTD_BOOL_FALSE
+}
+
 /// Checks if a slice contains `element`.
 /// Parameters:
 ///     `const NSTDSlice *const slice` - The slice.
-///     `const void *const element` - The element to search for.
-/// Returns: `NSTDInt32 is_in` - Nonzero if the slice contains `element`.
+///     `NSTDAnyConst element` - The element to search for.
+/// Returns: `NSTDBool is_in` - True if the slice contains `element`.
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_core_slice_contains(
     slice: &NSTDSlice,
-    element: *const c_void,
-) -> i32 {
-    let mut ptr = slice.data;
-    let element = core::slice::from_raw_parts(element as *const u8, slice.element_size);
+    element: NSTDAnyConst,
+) -> NSTDBool {
+    let mut ptr = slice.ptr.raw as *const u8;
+    let element = core::slice::from_raw_parts(element.cast(), slice.ptr.size);
     for _ in 0..slice.size {
-        let data = core::slice::from_raw_parts(ptr, slice.element_size);
+        let data = core::slice::from_raw_parts(ptr, slice.ptr.size);
         if data == element {
-            return 1;
+            return NSTDBool::NSTD_BOOL_TRUE;
         }
-        ptr = ptr.add(slice.element_size);
+        ptr = ptr.add(slice.ptr.size);
     }
-    0
+    NSTDBool::NSTD_BOOL_FALSE
 }
 
 /// Counts the number of `element`s found in `slice`.
 /// Parameters:
 ///     `const NSTDSlice *const slice` - The slice.
-///     `const void *const element` - The element to count.
+///     `NSTDAnyConst element` - The element to count.
 /// Returns: `NSTDUSize count` - The number of `element`s in `slice`.
 #[cfg_attr(feature = "clib", no_mangle)]
-pub unsafe extern "C" fn nstd_core_slice_count(slice: &NSTDSlice, element: *const c_void) -> usize {
-    let mut ptr = slice.data;
-    let element = core::slice::from_raw_parts(element.cast(), slice.element_size);
+pub unsafe extern "C" fn nstd_core_slice_count(slice: &NSTDSlice, element: NSTDAnyConst) -> usize {
+    let mut ptr = slice.ptr.raw as *const u8;
+    let element = core::slice::from_raw_parts(element.cast(), slice.ptr.size);
     let mut count = 0;
     for _ in 0..slice.size {
-        let data = core::slice::from_raw_parts(ptr, slice.element_size);
+        let data = core::slice::from_raw_parts(ptr, slice.ptr.size);
         if data == element {
             count += 1;
         }
-        ptr = ptr.add(slice.element_size);
+        ptr = ptr.add(slice.ptr.size);
     }
     count
 }
@@ -150,21 +167,21 @@ pub unsafe extern "C" fn nstd_core_slice_count(slice: &NSTDSlice, element: *cons
 /// Finds the first `element` in `slice` and returns the index of the element.
 /// Parameters:
 ///     `const NSTDSlice *const slice` - The slice.
-///     `const void *const element` - The element to search for.
+///     `NSTDAnyConst element` - The element to search for.
 /// Returns: `NSTDUSize index` - The index of the element, -1/usize::MAX if not found.
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_core_slice_find_first(
     slice: &NSTDSlice,
-    element: *const c_void,
+    element: NSTDAnyConst,
 ) -> usize {
-    let mut ptr = slice.data;
-    let element = core::slice::from_raw_parts(element.cast(), slice.element_size);
+    let mut ptr = slice.ptr.raw as *const u8;
+    let element = core::slice::from_raw_parts(element.cast(), slice.ptr.size);
     for i in 0..slice.size {
-        let data = core::slice::from_raw_parts(ptr, slice.element_size);
+        let data = core::slice::from_raw_parts(ptr, slice.ptr.size);
         if data == element {
             return i;
         }
-        ptr = ptr.add(slice.element_size);
+        ptr = ptr.add(slice.ptr.size);
     }
     usize::MAX
 }
@@ -172,21 +189,21 @@ pub unsafe extern "C" fn nstd_core_slice_find_first(
 /// Finds the last `element` in `slice` and returns the index of the element.
 /// Parameters:
 ///     `const NSTDSlice *const slice` - The slice.
-///     `const void *const element` - The element to search for.
+///     `NSTDAnyConst element` - The element to search for.
 /// Returns: `NSTDUSize index` - The index of the element, -1/usize::MAX if not found.
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_core_slice_find_last(
     slice: &NSTDSlice,
-    element: *const c_void,
+    element: NSTDAnyConst,
 ) -> usize {
-    let mut ptr = slice.end_unchecked().sub(slice.element_size);
-    let element = core::slice::from_raw_parts(element.cast(), slice.element_size);
+    let mut ptr = slice.end_unchecked().sub(slice.ptr.size);
+    let element = core::slice::from_raw_parts(element.cast(), slice.ptr.size);
     for i in (0..slice.size).rev() {
-        let data = core::slice::from_raw_parts(ptr, slice.element_size);
+        let data = core::slice::from_raw_parts(ptr, slice.ptr.size);
         if data == element {
             return i;
         }
-        ptr = ptr.sub(slice.element_size);
+        ptr = ptr.sub(slice.ptr.size);
     }
     usize::MAX
 }
@@ -195,44 +212,47 @@ pub unsafe extern "C" fn nstd_core_slice_find_last(
 /// Parameters:
 ///     `const NSTDSlice *const slice` - The slice.
 ///     `const NSTDSlice *const pattern` - The slice pattern.
-/// Returns: `NSTDInt32 starts_with` - Nonzero if `slice` starts with `pattern`.
+/// Returns: `NSTDBool starts_with` - True if `slice` starts with `pattern`.
 #[inline]
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_core_slice_starts_with(
     slice: &NSTDSlice,
     pattern: &NSTDSlice,
-) -> i32 {
+) -> NSTDBool {
     let slice = slice.as_byte_slice();
     let pattern = pattern.as_byte_slice();
-    slice.starts_with(pattern) as i32
+    NSTDBool::from(slice.starts_with(pattern))
 }
 
 /// Checks if a slice ends with another slice.
 /// Parameters:
 ///     `const NSTDSlice *const slice` - The slice.
 ///     `const NSTDSlice *const pattern` - The slice pattern.
-/// Returns: `NSTDInt32 ends_with` - Nonzero if `slice` ends with `pattern`.
+/// Returns: `NSTDBool ends_with` - True if `slice` ends with `pattern`.
 #[inline]
 #[cfg_attr(feature = "clib", no_mangle)]
-pub unsafe extern "C" fn nstd_core_slice_ends_with(slice: &NSTDSlice, pattern: &NSTDSlice) -> i32 {
+pub unsafe extern "C" fn nstd_core_slice_ends_with(
+    slice: &NSTDSlice,
+    pattern: &NSTDSlice,
+) -> NSTDBool {
     let slice = slice.as_byte_slice();
     let pattern = pattern.as_byte_slice();
-    slice.ends_with(pattern) as i32
+    NSTDBool::from(slice.ends_with(pattern))
 }
 
 /// Fills a slice with `element`.
 /// Parameters:
 ///     `NSTDSlice *const slice` - The slice.
-///     `const void *const element` - The element.
+///     `NSTDAnyConst element` - The element.
 #[inline]
 #[cfg_attr(feature = "clib", no_mangle)]
-pub unsafe extern "C" fn nstd_core_slice_fill(slice: &mut NSTDSlice, element: *const c_void) {
+pub unsafe extern "C" fn nstd_core_slice_fill(slice: &mut NSTDSlice, element: NSTDAnyConst) {
     nstd_core_slice_fill_range(
         slice,
         element,
         &NSTDURange {
             start: 0,
-            end: slice.size as u64,
+            end: slice.size,
         },
     );
 }
@@ -241,20 +261,21 @@ pub unsafe extern "C" fn nstd_core_slice_fill(slice: &mut NSTDSlice, element: *c
 /// NOTE: This function does NOT check that `range` is valid for operating on `slice`.
 /// Parameters:
 ///     `NSTDSlice *const slice` - The slice.
-///     `const void *const element` - The element.
+///     `NSTDAnyConst element` - The element.
 ///     `const NSTDURange *const range` - The range of the slice to fill.
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_core_slice_fill_range(
     slice: &mut NSTDSlice,
-    element: *const c_void,
+    element: NSTDAnyConst,
     range: &NSTDURange,
 ) {
-    let element = core::slice::from_raw_parts(element as *const u8, slice.element_size);
-    let mut ptr = slice.data.add(range.start as usize * slice.element_size);
+    let element = core::slice::from_raw_parts(element as *const u8, slice.ptr.size);
+    let start = range.start * slice.ptr.size;
+    let mut ptr = slice.ptr.raw.add(start).cast();
     for _ in range.start..range.end {
-        let data = core::slice::from_raw_parts_mut(ptr, slice.element_size);
+        let data = core::slice::from_raw_parts_mut(ptr, slice.ptr.size);
         data.copy_from_slice(element);
-        ptr = ptr.add(slice.element_size);
+        ptr = ptr.add(slice.ptr.size);
     }
 }
 
@@ -265,10 +286,10 @@ pub unsafe extern "C" fn nstd_core_slice_fill_range(
 ///     `const NSTDUSize j` - The index of the second element.
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_core_slice_swap(slice: &mut NSTDSlice, i: usize, j: usize) {
-    let i = slice.data.add(slice.element_size * i);
-    let j = slice.data.add(slice.element_size * j);
-    let slicei = core::slice::from_raw_parts_mut(i, slice.element_size);
-    let slicej = core::slice::from_raw_parts_mut(j, slice.element_size);
+    let i = slice.ptr.raw.add(slice.ptr.size * i);
+    let j = slice.ptr.raw.add(slice.ptr.size * j);
+    let slicei = core::slice::from_raw_parts_mut(i, slice.ptr.size);
+    let slicej = core::slice::from_raw_parts_mut(j, slice.ptr.size);
     slicei.swap_with_slice(slicej);
 }
 
@@ -277,13 +298,13 @@ pub unsafe extern "C" fn nstd_core_slice_swap(slice: &mut NSTDSlice, i: usize, j
 ///     `NSTDSlice *const slice` - The slice.
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_core_slice_reverse(slice: &mut NSTDSlice) {
-    let mut ptr = slice.data;
+    let mut ptr = slice.ptr.raw as *mut u8;
     let mut data = slice.as_byte_slice_mut();
     data.reverse();
     for _ in 0..slice.size {
-        data = core::slice::from_raw_parts_mut(ptr, slice.element_size);
+        data = core::slice::from_raw_parts_mut(ptr, slice.ptr.size);
         data.reverse();
-        ptr = ptr.add(slice.element_size);
+        ptr = ptr.add(slice.ptr.size);
     }
 }
 
@@ -294,7 +315,7 @@ pub unsafe extern "C" fn nstd_core_slice_reverse(slice: &mut NSTDSlice) {
 #[inline]
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_core_slice_shift_right(slice: &mut NSTDSlice, x: usize) {
-    let rot = x % slice.size * slice.element_size;
+    let rot = x % slice.size * slice.ptr.size;
     slice.as_byte_slice_mut().rotate_right(rot);
 }
 
@@ -305,7 +326,7 @@ pub unsafe extern "C" fn nstd_core_slice_shift_right(slice: &mut NSTDSlice, x: u
 #[inline]
 #[cfg_attr(feature = "clib", no_mangle)]
 pub unsafe extern "C" fn nstd_core_slice_shift_left(slice: &mut NSTDSlice, x: usize) {
-    let rot = x % slice.size * slice.element_size;
+    let rot = x % slice.size * slice.ptr.size;
     slice.as_byte_slice_mut().rotate_left(rot);
 }
 
@@ -348,6 +369,6 @@ pub unsafe extern "C" fn nstd_core_slice_swap_with_slice(s1: &mut NSTDSlice, s2:
 pub unsafe extern "C" fn nstd_core_slice_move(s1: &mut NSTDSlice, s2: &mut NSTDSlice) {
     const BYTE_SIZE: usize = core::mem::size_of::<u8>();
     nstd_core_slice_copy_from_slice(s1, s2);
-    let mut s2 = nstd_core_slice_new(s2.byte_count(), BYTE_SIZE, s2.data as *mut c_void);
-    nstd_core_slice_fill(&mut s2, &0u8 as *const u8 as *const c_void);
+    let mut s2 = nstd_core_slice_new(s2.byte_count(), BYTE_SIZE, s2.ptr.raw);
+    nstd_core_slice_fill(&mut s2, (&0u8 as *const u8) as NSTDAny);
 }
